@@ -125,6 +125,8 @@ fn ed_role(role: EditorType) -> Option<PersonRole> {
         EditorType::Reviser => None,
         EditorType::Collaborator => Some(PersonRole::Collaborator),
         EditorType::Organizer => Some(PersonRole::Organizer),
+        EditorType::Director => Some(PersonRole::Director),
+        EditorType::Unknown(role) => Some(PersonRole::Unknown(role)),
     }
 }
 
@@ -277,7 +279,14 @@ impl TryFrom<&tex::Entry> for Entry {
         }
 
         if let Some(title) = map_res(entry.title())?.map(Into::into) {
-            item.set_title(title);
+            if let Some(short_title) = map_res(entry.short_title())?.map(Into::into) {
+                item.set_title(FormatString {
+                    value: title,
+                    short: Some(Box::new(short_title)),
+                });
+            } else {
+                item.set_title(FormatString { value: title, short: None });
+            }
         }
 
         // NOTE: Ignoring subtitle and titleaddon for now
@@ -436,16 +445,18 @@ impl TryFrom<&tex::Entry> for Entry {
             item.set_url(QualifiedUrl { value: url, visit_date: date });
         }
 
-        if let Some(location) = map_res(entry.location())?.map(|d| d.into()) {
-            if let Some(parent) = book(&mut item, parent) {
-                parent.set_location(location);
-            } else {
-                item.set_location(location);
-            }
-        }
-
-        if let Some(publisher) = map_res(entry.publisher())?.map(|pubs| comma_list(&pubs))
+        if let Some(publisher_name) =
+            map_res(entry.publisher())?.map(|pubs| comma_list(&pubs))
         {
+            let location = map_res(entry.location())?.map(|d| d.into());
+            let publisher = Publisher::new(Some(publisher_name), location);
+            if let Some(parent) = book(&mut item, parent) {
+                parent.set_publisher(publisher);
+            } else {
+                item.set_publisher(publisher);
+            }
+        } else if let Some(location) = map_res(entry.location())?.map(|d| d.into()) {
+            let publisher = Publisher::new(None, Some(location));
             if let Some(parent) = book(&mut item, parent) {
                 parent.set_publisher(publisher);
             } else {
@@ -479,35 +490,21 @@ impl TryFrom<&tex::Entry> for Entry {
 
         if let Some(pages) = map_res(entry.pages())? {
             item.set_page_range(match pages {
-                PermissiveType::Typed(pages) => {
-                    if let Some(n) =
-                        pages.first().filter(|f| pages.len() == 1 && f.start == f.end)
-                    {
-                        MaybeTyped::Typed(Numeric::new(n.start as i32))
-                    } else {
-                        let mut items = vec![];
-                        for (i, pair) in pages.iter().enumerate() {
-                            let last = i + 1 == pages.len();
-                            let last_delim = (!last).then_some(NumericDelimiter::Comma);
-
-                            if pair.start == pair.end {
-                                items.push((pair.start as i32, last_delim));
+                PermissiveType::Typed(pages) => MaybeTyped::Typed(PageRanges::new(
+                    pages
+                        .into_iter()
+                        .map(|p| {
+                            if p.start == p.end {
+                                PageRangesPart::SinglePage(Numeric::from(p.start))
                             } else {
-                                items.push((
-                                    pair.start as i32,
-                                    Some(NumericDelimiter::Hyphen),
-                                ));
-                                items.push((pair.end as i32, last_delim));
+                                PageRangesPart::Range(
+                                    Numeric::from(p.start),
+                                    Numeric::from(p.end),
+                                )
                             }
-                        }
-
-                        MaybeTyped::Typed(Numeric {
-                            value: NumericValue::Set(items),
-                            prefix: None,
-                            suffix: None,
                         })
-                    }
-                }
+                        .collect(),
+                )),
                 PermissiveType::Chunks(chunks) => {
                     MaybeTyped::infallible_from_str(&chunks.format_verbatim())
                 }
@@ -524,12 +521,16 @@ impl TryFrom<&tex::Entry> for Entry {
             }
         }
 
+        if let Some(note) = map_res(entry.note())?.map(Into::into) {
+            item.set_note(note);
+        }
+
         if let Some(note) = map_res(entry.annotation())?
             .or_else(|| entry.addendum().ok())
-            .map(|d| d.format_verbatim())
+            .map(Into::into)
         {
             if item.note.is_none() {
-                item.set_note(note.into());
+                item.set_note(note);
             }
         }
 
@@ -537,10 +538,6 @@ impl TryFrom<&tex::Entry> for Entry {
             item.set_abstract_(abstract_.into())
         }
 
-        if let Some(annote) = map_res(entry.annotation())? {
-            item.set_annote(annote.into())
-        }
-
         if let Some(series) = map_res(entry.series())? {
             let title: FormatString = series.into();
             let mut new = Entry::new(&entry.key, item.entry_type);
